using namespace std;

#include <string.h>

#include "boardprint.h"
#include "boardstruc.h"

bool Jump(MarblePosType from, MarblePosType to, int depth);
bool JumpAscendingJ(int i, int j, int to, 
		    MarblePosType posTable[17][16], int depth);
bool JumpDescendingJ(int i, int j, int to,
		     MarblePosType posTable[17][16], int depth);

/* 0=free, 1=neutral zone, 2=own, 3=opponent */
int occupant[121];
int occupantTemplate[121] = {
  0,
  0, 0,
  0, 0, 0,
  0, 0, 0, 0,
  1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1,
  1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1,
  1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1,
  1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
  0, 0, 0, 0, 0, 0, 0, 0, 0,
  1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
  1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1,
  1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1,
  1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1,
  0, 0, 0, 0,
  0, 0, 0,
  0, 0,
  0
};

MarblePosType returnPath[121];
bool marked[121];
int returnPathLen;

// check whether there is a path between "from" and "to" and return one
// of the paths in "path" if so; path length is returned as function value;
// return path length of 0 if the move is not a jump; return path length of
// -1 if a path does not exist
int PathBetween(MarblePosType from, MarblePosType to, MarblePosType path[])
{
  int i, j;

  memcpy(occupant, occupantTemplate, sizeof(occupant));
  for (i = 0; i < 121; i++)
    marked[i] = false;
  marked[from] = true;	/* never give starting position as move */

  for (i = 0; i < 10; i++)
    occupant[currBoard[0][i]] = 2;

  for (i = 0; i < 10; i++)
    occupant[currBoard[1][i]] = 3;

  if (occupant[to] != 0)  // illegal move
    return -1;

  // set up of single move
  i = lineInHoriz[from];
  j = posInLineHoriz[from];
  if (j > 0 && occupant[horiz[i][j-1]] == 0 && to == horiz[i][j-1])
    return 0;
  if (j<posPerLine[i]-1 && occupant[horiz[i][j+1]]==0 && to==horiz[i][j+1])
    return 0;

  i = lineInSlash[from];
  j = posInLineSlash[from];
  if (j > 0 && occupant[slash[i][j-1]] == 0 && to == slash[i][j-1])
    return 0;
  if (j<posPerLine[i]-1 && occupant[slash[i][j+1]]==0 && to==slash[i][j+1])
    return 0;

  i = lineInBSlash[from];
  j = posInLineBSlash[from];
  if (j > 0 && occupant[bSlash[i][j-1]] == 0 && to == bSlash[i][j-1])
    return 0;
  if (j<posPerLine[i]-1 && occupant[bSlash[i][j+1]]==0 && to==bSlash[i][j+1])
    return 0;

  // once jump begins, position unoccupied, jumps unobstructed by its
  // nonexistent self
  occupant[from] = 0;
  
  if (Jump(from, to, 0)) {
    memcpy(path, returnPath, sizeof(returnPath));
    return returnPathLen;
  }

  return -1;
}

// basically a depth first search in six directions!
bool Jump(MarblePosType from, MarblePosType to, int depth)
{
  int i, j;
  
  i = lineInHoriz[from];
  j = posInLineHoriz[from];
  if (JumpAscendingJ(i, j, to, horiz, depth))
    return true;
  if (JumpDescendingJ(i, j, to, horiz, depth))
    return true;

  i = lineInSlash[from];
  j = posInLineSlash[from];
  if (JumpAscendingJ(i, j, to, slash, depth))
    return true;
  if (JumpDescendingJ(i, j, to, slash, depth))
    return true;

  i = lineInBSlash[from];
  j = posInLineBSlash[from];
  if (JumpAscendingJ(i, j, to, bSlash, depth))
    return true;
  if (JumpDescendingJ(i, j, to, bSlash, depth))
    return true;
}


bool JumpAscendingJ(int i, int j, int to, 
		    MarblePosType posTable[17][16], int depth)
{
  int k, l, bridgeLimit, landingPos;
  
  k = j+1;
  bridgeLimit = (posPerLine[i]-1 + j) / 2;
  while (k <= bridgeLimit && occupant[ posTable[i][k] ] < 2)
    k++;
  if (k <= bridgeLimit) {
    landingPos = k + (k - j);
    l = k+1;
    while (l <= landingPos && occupant[ posTable[i][l] ] < 2)
      l++;
    if (l > landingPos)
      if (! marked[ posTable[i][landingPos] ]) {
	marked[posTable[i][landingPos]] = true;
	if (posTable[i][landingPos] == to) {
	  returnPath[depth] = posTable[i][landingPos];
	  returnPathLen = depth+1;
	  return true;
	}
	else {
	  returnPath[depth] = posTable[i][landingPos];
	  return(Jump(posTable[i][landingPos], to, depth+1));
	}
      }
  }
  return false;
}

bool JumpDescendingJ(int i, int j, int to,
		     MarblePosType posTable[17][16], int depth)
{
  int k, l, bridgeLimit, landingPos;
  
  k = j-1;
  bridgeLimit = (j+1) / 2;
  while (k >= bridgeLimit && occupant[ posTable[i][k] ] < 2)
    k--;
  if (k >= bridgeLimit) {
    landingPos = k - (j - k);
    l = k-1;
    while (l >= landingPos && occupant[ posTable[i][l] ] < 2)
      l--;
    if (l < landingPos)
      if (! marked[ posTable[i][landingPos] ]) {
	marked[posTable[i][landingPos]] = true;
	if (posTable[i][landingPos] == to) {
	  returnPath[depth] = posTable[i][landingPos];
	  returnPathLen = depth+1;
	  return true;
	}
	else {
	  returnPath[depth] = posTable[i][landingPos];
	  return(Jump(posTable[i][landingPos], to, depth+1));
	}
      }
  }
  return false;
}

void MakeMove(int player, int from, int to)
{
  for (int i = 0; i < 10; i++)
    if (currBoard[player-1][i] == from) {
      currBoard[player-1][i] = to;
      break;
    }
}

bool CheckAndMakeMove(int player, int from, int to, MarblePosType path[121],
		      int &pathLen)
{
  pathLen = PathBetween(from, to, path);
  if (pathLen >= 0) {
    MakeMove(player, from, to);
    return true;
  }
  else
    return false;
}

bool HasWon(int player)
{
  int i;

  if (player == 1) {
    bool goneHome[10];

    for (i = 0; i < 10; i++)
      goneHome[i] = false;

    for (i = 0; i < 10; i++)
      if (111 <= currBoard[0][i] && currBoard[0][i] <= 120)
	goneHome[ currBoard[0][i] - 111 ] = true;

    bool finished = true;
    for (i = 0; i < 10; i++)
      if (!goneHome[i])
	finished = false;
    return finished;
  }
  else {
    bool goneHome[10];

    for (i = 0; i < 10; i++)
      goneHome[i] = false;

    for (i = 0; i < 10; i++)
      if (0 <= currBoard[1][i] && currBoard[1][i] < 10)
	goneHome[ currBoard[1][i] ] = true;

    bool finished = true;
    for (i = 0; i < 10; i++)
      if (!goneHome[i])
	finished = false;
    return finished;
  }
}
