using namespace std;

#include <iostream>
#include <fstream>
#include <sstream>
#include <unistd.h>
#include <assert.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

pid_t pid1, pid2;

ifstream *in1, *in2;  // streams parent uses to communicate w/ players
ofstream *out1, *out2;

void InitPlayers(char *player1, char *player2)
{
  int p1i[2], p1o[2], p2i[2], p2o[2];

  assert(!pipe(p1i));
  assert(!pipe(p1o));
  assert(!pipe(p2i));
  assert(!pipe(p2o));

  // Create child 1
  pid1 = fork();
  assert(pid1 >= 0);

  if (pid1 == pid_t(0)) {    // process for child 1
    close(p1i[0]);
    close(p1o[1]);
    assert(dup2(p1i[1], STDOUT_FILENO) >= 0);
    close(p1i[1]);
    assert(dup2(p1o[0], STDIN_FILENO) >= 0);
    close(p1o[0]); 
    assert(execl(player1, player1, NULL) != -1);
  }
  else {     // continuing process for parent
    pid2 = fork();
    assert(pid2 >= 0);

    if (pid2 == pid_t(0)) {    // process for child 2
      close(p2i[0]);
      close(p2o[1]);
      assert(dup2(p2i[1], STDOUT_FILENO) >= 0);
      close(p2i[1]);
      assert(dup2(p2o[0], STDIN_FILENO) >= 0);
      close(p2o[0]); 
      assert(execl(player2, player2, NULL) != -1);
    }
    else {
      close(p1i[1]);
      close(p1o[0]);

      in1 = new ifstream(p1i[0]);
      out1 = new ofstream(p1o[1]);

      close(p2i[1]);
      close(p2o[0]);

      in2 = new ifstream(p2i[0]);
      out2 = new ofstream(p2o[1]);
    }
  }
}

void KillPlayers()
{
  assert(!kill(pid1, SIGKILL));
  assert(!kill(pid2, SIGKILL));
}

void SendInitialMessages()
{
  *out1 << "1\n" << flush;
  *out2 << "2\n" << flush;
}

void ReadMove(int player, int &from, int &to)
{
  if (player == 1) {
    *in1 >> from >> to;
    from--;
    to--;
  }
  else {
    *in2 >> from >> to;
    from = 121 - from;
    to = 121 - to;
  }
}

void SendMove(int player, int from, int to)
{
  if (player == 1)
    *out1 << from + 1 << " " << to + 1 << endl;
  else
    *out2 << 121 - from << " " <<  121 - to << endl;
}

long GetTimeUsed(int player)
{
  // Note: this depends highly on Linux's /proc pseudo filesystem

  ostrstream procFN;
  if (player == 1)
    procFN << "cat /proc/" << pid1 << "/stat" << '\0';
  else
    procFN << "cat /proc/" << pid2 << "/stat" << '\0';

  FILE *procFile = popen(procFN.str(), "r");

  int utime, stime;
  fscanf(procFile, "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*u %*u %*u %*u %d %d", &utime, &stime);

  fclose(procFile);
  return utime + stime;
}


